/****************************************************************************
* MeshLab                                                           o o     *
* A versatile mesh processing toolbox                             o     o   *
*                                                                _   O  _   *
* Copyright(C) 2005                                                \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include <QtGui>

#include <math.h>
#include <limits>
#include <stdlib.h>
#include "fgt_decorate_20121130.h"
#include <QGLWidget>
#include <wrap/gl/addons.h>

#include <meshlab/glarea.h>
#include <common/pluginmanager.h>
#include <vcg/complex/algorithms/intersection.h>
#include <wrap/io_trimesh/export_ply.h>
using namespace vcg;
using namespace std;

class MyFace;
class MyEdge;
class MyVertex;

struct MyUsedTypes : public UsedTypes<	Use<MyVertex>		::AsVertexType,
																				Use<MyEdge>			::AsEdgeType,
																				Use<MyFace>			::AsFaceType>{};


class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::BitFlags, vertex::Normal3f, vertex::Mark>{};
class MyEdge    : public Edge< MyUsedTypes, edge::VertexRef, edge::EVAdj> {};
class MyFace    : public Face  <MyUsedTypes, face::VertexRef,face::FFAdj, face::BitFlags, face::Normal3f> {};

class MyEdgeMesh: public tri::TriMesh< vector<MyVertex>, vector<MyEdge> > {};

typedef std::pair<Point3f,Color4b> PointPC; // this type is used to have a simple coord+color pair to rapidly draw non manifold faces


void DrawLineVector(std::vector<PointPC> &EV)
{
  glPushAttrib(GL_ENABLE_BIT|GL_VIEWPORT_BIT| GL_CURRENT_BIT | GL_DEPTH_BUFFER_BIT);

  glDisable(GL_LIGHTING);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_LINE_SMOOTH);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glLineWidth(1.f);
  glDepthRange (0.0, 0.999);
  glEnableClientState (GL_VERTEX_ARRAY);
  glEnableClientState (GL_COLOR_ARRAY);

  glVertexPointer(3,GL_FLOAT,sizeof(PointPC),&(EV.begin()[0].first));
  glColorPointer(4,GL_UNSIGNED_BYTE,sizeof(PointPC),&(EV.begin()[0].second));
  glDrawArrays(GL_LINES,0,EV.size());

  glDisableClientState (GL_COLOR_ARRAY);
  glDisableClientState (GL_VERTEX_ARRAY);

  glPopAttrib();
}




QString DecorateBackgroundPlugin::decorationInfo(FilterIDType id) const
{
  switch(id)
	{
	case DP_SHOW_SECTION :      return tr("Show a planar section of a mesh");
	}
	assert(0);
	return QString();
}
QString DecorateBackgroundPlugin::decorationName(FilterIDType id) const
{
	switch(id)
	{
	case DP_SHOW_SECTION :      return tr("Show Planar Section");
	}
	assert(0);
	return QString();
}

void DecorateBackgroundPlugin::initGlobalParameterSet(QAction *action, RichParameterSet &parset)
{
  switch(ID(action)){
  case DP_SHOW_SECTION :
    parset.addParam(new RichFloat(HeightParam(),0.0,"P","Where on the Z axis the plane intersect the mesh"));
    parset.addParam(new RichPoint3f(PointParam(),Point3f(0,0,0),"Plane passthrough","Where on the Z axis the plane intersect the mesh"));
  break;
  }
}		
		
bool DecorateBackgroundPlugin::startDecorate( QAction * action, MeshModel &m, RichParameterSet * parset, GLArea * gla)
{
  switch(ID(action)){
    case DP_SHOW_SECTION :
  {
    float h = 0;//parset->getFloat(this->HeightParam());
    Point3f pass = parset->getPoint3f(this->PointParam());

    Plane3f plane(pass[2],Point3f(0,0,1));

    MyEdgeMesh edge_mesh;  // returned EdgeMesh (i.e. the cross-section)

    IntersectionPlaneMesh<CMeshO, MyEdgeMesh, float>(m.cm, plane, edge_mesh);
    qDebug("Computed a section of %i edges",edge_mesh.EN());

    CMeshO::PerMeshAttributeHandle< vector<PointPC> > bvH =
        tri::Allocator<CMeshO>::GetPerMeshAttribute< vector<PointPC> >(m.cm,"PlanarSectionVertexVector");
    if(!tri::Allocator<CMeshO>::IsValidHandle(m.cm,bvH))
      bvH=tri::Allocator<CMeshO>::AddPerMeshAttribute< vector<PointPC> >(m.cm,string("PlanarSectionVertexVector"));
    vector<PointPC> *BVp = &bvH();
    BVp->clear();
    Color4b cc=Color4b::Red;
    for(MyEdgeMesh::EdgeIterator ei=edge_mesh.edge.begin(); ei != edge_mesh.edge.end();++ei)
    {
      BVp->push_back(make_pair(ei->V(0)->cP(),cc));
      BVp->push_back(make_pair(ei->V(1)->cP(),cc));
    }
  }
    break;
  }
	return true;
}

void DecorateBackgroundPlugin::decorateMesh(QAction *a, MeshModel &m, RichParameterSet * parset,GLArea *gla, QPainter *, GLLogStream &)
{
  switch(ID(a)){
    case DP_SHOW_SECTION :
  {
    CMeshO::PerMeshAttributeHandle< vector<PointPC> > bvH =
        tri::Allocator<CMeshO>::GetPerMeshAttribute< vector<PointPC> >(m.cm,"PlanarSectionVertexVector");
    DrawLineVector(bvH());
   }
  }
}

Q_EXPORT_PLUGIN(DecorateBackgroundPlugin)
